#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
# Andre Anjos <andre.anjos@idiap.ch>
# Wed 11 May 2011 12:05:19 CEST 

"""Plots a single (or combined) DET curve given input score files.
"""

__epilog__ = """
Examples:

  1. Plot a single DET curve with no extra options:

     $ plotDET.py scores.txt

     The output should now be in the file `det.pdf`.

  2. Set title and output file (format chosen using output filename extension):

     $ plotDET.py --tile="My Beautiful DET Curve" --output=test.png scores.txt

  3. Plot multiple curves in a single plot:

     $ plotDET.py --title="My Test DET" --output=overlayed.pdf --label=devel development.scores --label=test test.scores

  4. Fine-tune zooming:

     $ plotDET.py --limits=80 99.98 80 99.98 scores.txt
"""

import os
import sys

def get_options():
  """Processes user options and arguments, makes sure all is ok."""
  import optparse

  class MyParser(optparse.OptionParser):
    def format_epilog(self, formatter):
      return self.epilog

  usage = 'usage: %s [options] <score-file> [+<score-file>]' % os.path.basename(sys.argv[0])

  parser = MyParser(usage=usage, description=__doc__, epilog=__epilog__)

  desiredLabels = ["0.001","0.002","0.005","0.01","0.02","0.05","0.1","0.2","0.5","1","2","5","10","20","40","60","80","90","95","98","99","99.5","99.8","99.9","99.95","99.98","99.99","99.995","99.998","99.999"]

  parser.add_option('-n', '--number-of-points', dest="points", default=100, help="number of points to use for each curve on the plot (defaults to %default)", metavar="INT", type="int")
  parser.add_option('-c', '--colour', dest="colour", default=False, action="store_true", help="set this flag if you would like to have a coloured plot instead of the default gray-scale.")
  parser.add_option('-l', '--limits', nargs=4, dest="limits", default=("0.001", "99.999", "0.001", "99.999"), help="axis limits for plot in the format: min-FRR max-FRR min-FAR max-FRR (use white spaces between values). Values have to be selected among available choices: 0.001,0.002,0.005,0.01,0.02,0.05,0.1,0.2,0.5,1,2,5,10,20,40,60,80,90,95,98,99,99.5,99.8,99.9,99.95,99.98,99.99,99.995,99.998,99.999. (defaults to %default)", choices=desiredLabels)
  parser.add_option('-t', '--title', dest="title", default="",
      help="optional plot title, leave empty for no title display (defaults to "'%default")')
  parser.add_option('-b', '--label', dest="labels", default=None, action="append", help="optional labels to be set in each of the lines drawn. If labels are given, there should be one label for each input score file. To specify multiple lables, call this option as many times as required (e.g. -b label1 -b label2). If no labels are given, no legend will be drawn")
  parser.add_option('-o', '--output', dest="output", default="det.pdf",
      help="output filename to be used. The file format will be chosen according to the extension (supported: all supported by matplotlib)")

  options, args = parser.parse_args()

  if len(args) == 0:
    parser.error("you should give at least a single score-file to process")

  if isinstance(options.labels, str): options.labels = [options.labels]

  if options.labels is not None:
    if len(args) != len(options.labels):
      parser.error("the number of label arguments should be the same as the number of score-files given (%d != %d)" % (len(options.labels), len(args)))

  return options, args

def main():
  """The main method, just process options and call STK"""
 
  from .. import split, load_file, plotDET

  options, args = get_options()

  # get negatives and positive scores
  negatives = []
  positives = []
  for k in args:
    print("Loading score file %s..." % k)
    neg, pos = split(load_file(k, no_labels=True))
    negatives.append(neg)
    positives.append(pos)

  plotDET(negatives, positives, filename=options.output,
    points=options.points, limits=options.limits, title=options.title,
    labels=options.labels, colour=options.colour)

  sys.exit(0)
