#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
# Andre Anjos <andre.anjos@idiap.ch>
# Wed 11 May 2011 12:05:19 CEST 

"""Plots a combined score plot with impostors, clients and attacks
"""

__epilog__ = """
Examples:

  1. Plot a single score (bar) plot

     $ plotScores.py legit.txt attack.txt

     The output should now be in the file `scores.pdf`.

  2. Set title and output file (format chosen using output filename extension):

     $ plotDET.py --title="My Beautiful Score plot" --output=test.png legit.txt attack.txt
"""

import os
import sys

def get_options():
  """Processes user options and arguments, makes sure all is ok."""
  import optparse

  class MyParser(optparse.OptionParser):
    def format_epilog(self, formatter):
      return self.epilog

  usage = 'usage: %s [options] legit.txt [attack.txt]' % os.path.basename(sys.argv[0])

  parser = MyParser(usage=usage, description=__doc__, epilog=__epilog__)

  parser.add_option('-c', '--colour', dest="colour", default=False, action="store_true", help="set this flag if you would like to have a coloured plot instead of the default gray-scale.")
  parser.add_option('-t', '--title', dest="title", default="",
      help="optional plot title, leave empty for no title display (defaults to "'%default")')
  parser.add_option('-b', '--label', dest="labels", default=None, action="append", help="optional labels to be set in each of the histograms drawn. If labels are given, there should be one label for each input score file. To specify multiple lables, call this option as many times as required (e.g. -b label1 -b label2). If no labels are given, no legend will be drawn")
  parser.add_option('-B', '--bins', dest="bins", default=None, action="append",
      type='int', help="optional number of bins to use for each histogram drawn. If not set, will use the matplotlib default")
  parser.add_option('-o', '--output', dest="output", default="scores.pdf",
      help="output filename to be used. The file format will be chosen according to the extension (supported: all supported by matplotlib)")

  options, args = parser.parse_args()

  if len(args) < 1 or len(args) > 2:
    parser.error("you should give at least a single score-file or at most 2, which account for the legit access protocol and the attack protocol files")

  if isinstance(options.labels, str): options.labels = [options.labels]

  if options.labels is not None:
    if (len(args) == 1 and len(options.labels) != 2) or \
        (len(args) == 2 and len(options.labels) != 3):
      parser.error("the number of label arguments should be 2 if you pass only 1 score file and 3 in case you pass two score files")

  if options.bins is not None:
    if (len(args) == 1 and len(options.bins) != 2) or \
        (len(args) == 2 and len(options.bins) != 3):
      parser.error("the number of bins arguments should be 2 if you pass only 1 score file and 3 in case you pass two score files")

  return options, args

def main():
  """The main method, just process options and call STK"""
 
  # massages the PYTHONPATH so we get the `stk` package on it.
  sys.path.insert(0, os.path.dirname(os.path.realpath(sys.argv[0])))

  import stk

  options, args = get_options()

  # get negatives and positive scores
  if len(args) >= 1:
    print("Loading legit access score file %s..." % args[0])
    impostors, clients = stk.split(stk.load_file(args[0], no_labels=True))

  if len(args) == 2:
    print("Loading attacks score file %s..." % args[1])
    attacks, discard = stk.split(stk.load_file(args[1], no_labels=True))
    print("Discarding real-accesses, keeping only attacks...")
    del discard

  data = [impostors, clients]
  if len(args) == 2:
    data.append(attacks)

  stk.plotScores(data, filename=options.output, bins=options.bins,
    title=options.title, labels=options.labels, colour=options.colour)

  sys.exit(0)
